!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
subroutine check_exists(file)
  !
  ! Check and make sure 'file' exists
  !
  character(len=*),intent(in)::file
  !
  logical::exists
  !
  inquire(file=trim(file),exist=exists)
  if (.not.(exists)) then
     write(*,'(a,'' not found. STOP.'')') trim(file)
     call abort()
  endif
end subroutine check_exists
